#include <psychlops.h>
using namespace Psychlops;


//reference:
//McCollough, C. (1965). Adaptation of edge-detectors in the human visual system. Science, 149, 1115-1116.

void McCollough() {
       double bg_lum    = 0.2;
       int mode = 0;

       int W = Display::getWidth();
       int H = Display::getHeight();
       int w = W;
       int h = H;

       int sCycle = 50;
       double tf = 1;
       double sf = 0.03;
       double t;

       int frame = 0;

       Psychlops::Point p1,p2;

       Range rng;
       Independent << mode | "adaptation/test1/test2"      |   0<= rng<= 1 | 1 | 1 ;
       Independent << tf  | "temporal frequency [Hz]" | 0.0<=rng<=100.0 |  1 | 0.1;

       while(!Input::get(Keyboard::esc)) {
               Display::clear(Color(bg_lum));
               sf = (double)1/sCycle;
               t = (double)frame/Display::getRefreshRate();

               if(mode == 0){
                       for(int i=-w/2; i<w/2; i++){
                               p1.set(w/2+i,0);
                               p2.set(w/2+i,h);
                               Display::line(p1,p2,(W/2+i)%(sCycle)<sCycle/2?(Psychlops::Color::green):(Psychlops::Color::black));
                       }
                       Display::flip(Display::getRefreshRate()/tf/2);
                       for(int j=-h/2; j<h/2; j++){
                               p1.set(0,h/2+j);
                               p2.set(w,h/2+j);
                               Display::line(p1,p2,(H/2+j)%(sCycle)<sCycle/2?(Psychlops::Color::red):(Psychlops::Color::black));
                       }
                       Display::flip(Display::getRefreshRate()/tf/2);
               }else if(mode == 1){
                       for(int i=0; i<w/2; i++){
                               p1.set(i,h/2);
                               p2.set(i,h);
                               Display::line(p1,p2,(W/2+i)%(sCycle)<sCycle/2?(Psychlops::Color::white):(Psychlops::Color::black));
                               p1.set(w/2+i,0);
                               p2.set(w/2+i,h/2);
                               Display::line(p1,p2,(W/2+i)%(sCycle)<sCycle/2?(Psychlops::Color::white):(Psychlops::Color::black));
                       }
                       for(int j=0; j<h/2; j++){
                               p1.set(0,j);
                               p2.set(w/2,j);
                               Display::line(p1,p2,(H/2+j)%(sCycle)<sCycle/2?(Psychlops::Color::white):(Psychlops::Color::black));
                               p1.set(w/2,h/2+j);
                               p2.set(w,h/2+j);
                               Display::line(p1,p2,(H/2+j)%(sCycle)<sCycle/2?(Psychlops::Color::white):(Psychlops::Color::black));
                       }
                       Display::flip(1);
               }
       }
}


void psychlops_main() {
       Canvas display(Canvas::fullscreen);

       Procedure p;
       p.setDesign(Procedure::DEMO);
       p.setProcedure(McCollough);
       p.run();

}

